/*
 * MudStorm OS
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef __SYSTEM_H__
#define __SYSTEM_H__

// TYPES.H
#if !defined(NULL)
#define NULL ((void*)0)
#endif

#if !defined(EOF)
#define EOF (-1)
#endif

#if !defined(true)
#define true 1
#endif

#if !defined(false)
#define false 0
#endif

#if !defined(TRUE)
#define TRUE 1
#endif

#if !defined(FALSE)
#define FALSE 0
#endif

#if !defined(UINT32_MAX)
#define UINT32_MAX  (0xffffffff)
#endif

#define BLACK 0
#define BLUE 1
#define GREEN 2
#define CYAN 3
#define RED 4
#define MAGENTA 5
#define BROWN 6
#define GREY 7
#define DARK_GREY 8
#define BRIGHT_BLUE 9
#define BRIGHT_GREEN 10
#define BRIGHT_CYAN 11
#define BRIGHT_RED 12
#define BRIGHT_MAGENTA 13
#define YELLOW 14
#define WHITE 15
#define BAK_WHITE 0xF0

typedef char* string;

typedef int bool;

typedef unsigned int size_t;

typedef unsigned char byte;
typedef unsigned short int word;
typedef unsigned long dword;
typedef unsigned long long qword; //8 byte

typedef signed char int8_t;
typedef signed short int16_t;	/* Note: multi-byte values are little-endian */
typedef signed long int32_t;
typedef signed long int64_t;

typedef unsigned char uint8_t;
typedef unsigned short uint16_t;	/* Note: multi-byte values are little-endian */
typedef unsigned long uint32_t;
typedef unsigned long uint64_t;

typedef unsigned long long cycles_t;

// KERNEL_MAIN.C
extern void initShell();

// CONSOLE.c
extern void scroll(void);
extern void shell_csr_fix();
extern void cls();
extern void pcolor(char);
extern void putch(char);
extern void puts(char*);
extern void move_csr(void);
extern unsigned char inportb(unsigned short);
extern unsigned short inportw(unsigned short);
extern void outportb(unsigned short, unsigned char);
extern void outportw(unsigned short, unsigned short);
extern void shutdown();
extern void reboot();
extern void logo();

// LOAD.C
extern void Init(unsigned int);

// LOGIN.C
extern bool login(string, string);

// RANDOM.C
extern int rand(void);
extern void srand(unsigned int);

// STRING.C
extern string strcat (string, const string);
extern int strlen(const char*);
extern int strcmp(const char*, const char*);
extern int strncmp (const char*, const char*, const unsigned int);
extern char *strncpy (char*, register const char*, register size_t);
extern char *strcpy(char*, const char*);
extern void memset(void*, int, size_t);
extern void memmov (void*, const void*, const unsigned int);
extern void *memcpy(void*, const void*, size_t);
extern unsigned short *memsetw(unsigned short*, unsigned short, size_t);
extern int stoi(char*);
extern int isdigit(int);

// SPEAKER.C
extern void sound(int);
extern void nosound();
extern void beep();

// ISR.C
struct regs
{
    unsigned int gs, fs, es, ds;
    unsigned int edi, esi, ebp, esp, ebx, edx, ecx, eax;
    unsigned int int_no, err_code;
    unsigned int eip, cs, eflags, useresp, ss;    
};

// GDT.C
extern void gdt_set_gate(int num, unsigned long base, unsigned long limit, unsigned char access, unsigned char gran);
extern void gdt_install();

// IDT.C
extern void idt_set_gate(unsigned char num, unsigned long base, unsigned short sel, unsigned char flags);
extern void idt_install();

// ISRS.C
extern void isrs_install();

// IRQ.C
extern void irq_install_handler(int irq, void (*handler)(struct regs *r));
extern void irq_uninstall_handler(int irq);
extern void irq_install();

// TIMER.C
extern void wait(int);
extern void timer_install();

// RDTSC.C
extern cycles_t currentcycles();

// KB.C
extern void keyboard_install();

// SHELL.C
extern void runShell();

#define NUM 11

struct cmd {
	char* cmdname;
	void (*v_func)(void);
	char* cmddesc;
};

extern struct cmd shellcmd[NUM];

#endif
