/*
 * MudStorm OS
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <system.h>

unsigned short *memsetw(unsigned short *dest, unsigned short val, size_t count)
{
    	unsigned short *temp = (unsigned short *)dest;
    	for( ; count != 0; count--) *temp++ = val;
    	return dest;
}

void *memcpy(void *dst, const void *src, size_t len)
    /// Copy block of memory from src to dst, len bytes long
{
    char *sp = (char*) src;
    char *dp = (char*) dst;
    for(; len != 0; len--) *dp++ = *sp++;
	return dst;
}

void memset(void *dst, int val, size_t len)
{
    char *temp = (char*) dst;
    for ( ; len != 0; len--) *temp++ = val;
}

void memmov (void *dst, const void *src, const unsigned int bytes)
{
    unsigned int i = 0;

    for (i=0; i<bytes; i++) {
        ((char *)dst)[i] = ((char *)src)[i];
    }
}

string strcat (string dest, const string src)
{
	int dest_len = strlen(dest), src_len = strlen(src), i = 0;

	for (i=0;i<(dest_len+src_len);i++)
	{
		dest[dest_len + i] = src[i];
		i++;
	}
	dest[dest_len + src_len] = '\0';
	return dest;
}

int strlen(const char *str) 
{
	int i = 0;
	while(str[i] != '\0') 
	{
		++i;
	}
	return i;
}

int strcmp(const char *s1, const char *s2)
    /// Compare two null ended strings
{
    for(; *s1 == *s2; ++s1, ++s2)
        if(*s1 == 0)
            return 0;
    return *(unsigned char *)s1 < *(unsigned char *)s2 ? -1 : 1;
}

int strncmp (const char *dst, const char *src, const unsigned int bytes) 
{
    unsigned int i = 0;
    int result = 0;

    for (i=0; i<bytes; i++) {
        if (dst[i] < src[i]) {
            result = -1;
            break;
        } else if (dst[i] > src[i]) {
            result = 1;
            break;
        }
    }

    return result;
}

char *strncpy (char *dest, register const char *src, register size_t n)
{
  if (n != 0) {
	char *d = dest;
	const char *s = src;
	do {
		if ((*d++ = *s++) == 0) {
			while (--n != 0)
				*d++ = '\0';
				break;
			}
		} while (--n != 0);
	}
	return dest;
 }

char *strcpy(char *dst, const char *src)
{
    char c;
    char *p = dst;

    while ((c = *src++))
    {
        *p++ = c;
    }

    *p = '\0';
    return dst;
}

int stoi(char *str)
{
  int result;
  int puiss;

  result = 0;
  puiss = 1;
  while (('-' == (*str)) || ((*str) == '+'))
    {
      if (*str == '-')
        puiss = puiss * -1;
      str++;
    }
  while ((*str >= '0') && (*str <= '9'))
    {
      result = (result * 10) + ((*str) - '0');
      str++;
    }
  return (result * puiss);
}

int isdigit(int c) {
  if (c >= 48 && c <= 57)
    return 1;
  return 0;
}
